@@ -17,29 +17,34 @@ from collections import namedtuple
 
 from pyfakefs.tests.test_utils import RealFsTestCase
 
-FileTime = namedtuple('FileTime', 'st_ctime, st_atime, st_mtime')
+FileTime = namedtuple("FileTime", "st_ctime, st_atime, st_mtime")
 
 
 class FakeStatTestBase(RealFsTestCase):
-
     def setUp(self):
         super().setUp()
         # we disable the tests for MacOS to avoid very long builds due
         # to the 1s time resolution - we know that the functionality is
         # similar to Linux
         self.check_linux_and_windows()
-        self.file_path = self.make_path('some_file')
+        self.file_path = self.make_path("some_file")
         # MacOS has a timestamp resolution of 1 second
         self.sleep_time = 1.1 if self.is_macos else 0.01
-        self.mode = ''
+        self.mode = ""
 
     def stat_time(self, path):
         stat = self.os.stat(path)
-        # sleep a bit so in the next call the time has changed
-        time.sleep(self.sleep_time)
-        return FileTime(st_ctime=stat.st_ctime,
-                        st_atime=stat.st_atime,
-                        st_mtime=stat.st_mtime)
+        if self.use_real_fs():
+            # sleep a bit so in the next call the time has changed
+            time.sleep(self.sleep_time)
+        else:
+            # calling time.time() advances mocked time
+            time.time()
+        return FileTime(
+            st_ctime=stat.st_ctime,
+            st_atime=stat.st_atime,
+            st_mtime=stat.st_mtime,
+        )
 
     def assertLessExceptWindows(self, time1, time2):
         if self.is_windows_fs:
@@ -54,103 +59,111 @@ class FakeStatTestBase(RealFsTestCase):
             self.assertEqual(time1, time2)
 
     def open_close_new_file(self):
-        with self.open(self.file_path, self.mode):
-            created = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
-
-        return created, closed
+        with self.mock_time():
+            with self.open(self.file_path, self.mode):
+                created = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
+            return created, closed
 
     def open_write_close_new_file(self):
-        with self.open(self.file_path, self.mode) as f:
-            created = self.stat_time(self.file_path)
-            f.write('foo')
-            written = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+        with self.mock_time():
+            with self.open(self.file_path, self.mode) as f:
+                created = self.stat_time(self.file_path)
+                f.write("foo")
+                written = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
         return created, written, closed
 
     def open_close(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, self.mode):
-            opened = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, self.mode):
+                opened = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, closed
+            return before, opened, closed
 
     def open_write_close(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, self.mode) as f:
-            opened = self.stat_time(self.file_path)
-            f.write('foo')
-            written = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, self.mode) as f:
+                opened = self.stat_time(self.file_path)
+                f.write("foo")
+                written = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, written, closed
+            return before, opened, written, closed
 
     def open_flush_close(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, self.mode) as f:
-            opened = self.stat_time(self.file_path)
-            f.flush()
-            flushed = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, self.mode) as f:
+                opened = self.stat_time(self.file_path)
+                f.flush()
+                flushed = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, flushed, closed
+            return before, opened, flushed, closed
 
     def open_write_flush(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, self.mode) as f:
-            opened = self.stat_time(self.file_path)
-            f.write('foo')
-            written = self.stat_time(self.file_path)
-            f.flush()
-            flushed = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, self.mode) as f:
+                opened = self.stat_time(self.file_path)
+                f.write("foo")
+                written = self.stat_time(self.file_path)
+                f.flush()
+                flushed = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, written, flushed, closed
+            return before, opened, written, flushed, closed
 
     def open_read_flush(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, 'r') as f:
-            opened = self.stat_time(self.file_path)
-            f.read()
-            read = self.stat_time(self.file_path)
-            f.flush()
-            flushed = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, "r") as f:
+                opened = self.stat_time(self.file_path)
+                f.read()
+                read = self.stat_time(self.file_path)
+                f.flush()
+                flushed = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, read, flushed, closed
+            return before, opened, read, flushed, closed
 
     def open_read_close_new_file(self):
-        with self.open(self.file_path, self.mode) as f:
-            created = self.stat_time(self.file_path)
-            f.read()
-            read = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+        with self.mock_time():
+            with self.open(self.file_path, self.mode) as f:
+                created = self.stat_time(self.file_path)
+                f.read()
+                read = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return created, read, closed
+            return created, read, closed
 
     def open_read_close(self):
-        self.create_file(self.file_path)
+        with self.mock_time():
+            self.create_file(self.file_path)
 
-        before = self.stat_time(self.file_path)
-        with self.open(self.file_path, self.mode) as f:
-            opened = self.stat_time(self.file_path)
-            f.read()
-            read = self.stat_time(self.file_path)
-        closed = self.stat_time(self.file_path)
+            before = self.stat_time(self.file_path)
+            with self.open(self.file_path, self.mode) as f:
+                opened = self.stat_time(self.file_path)
+                f.read()
+                read = self.stat_time(self.file_path)
+            closed = self.stat_time(self.file_path)
 
-        return before, opened, read, closed
+            return before, opened, read, closed
 
     def check_open_close_new_file(self):
         """
@@ -376,7 +389,7 @@ class FakeStatTestBase(RealFsTestCase):
 class TestFakeModeW(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeW, self).setUp()
-        self.mode = 'w'
+        self.mode = "w"
 
     def test_open_close_new_file(self):
         self.check_open_close_new_file()
@@ -397,7 +410,7 @@ class TestFakeModeW(FakeStatTestBase):
         self.check_open_write_flush_close_w_mode()
 
     def test_read_raises(self):
-        with self.open(self.file_path, 'w') as f:
+        with self.open(self.file_path, "w") as f:
             with self.assertRaises(OSError):
                 f.read()
 
@@ -410,7 +423,7 @@ class TestRealModeW(TestFakeModeW):
 class TestFakeModeWPlus(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeWPlus, self).setUp()
-        self.mode = 'w+'
+        self.mode = "w+"
 
     def test_open_close_new_file(self):
         self.check_open_close_new_file()
@@ -463,7 +476,7 @@ class TestRealModeWPlus(TestFakeModeWPlus):
 class TestFakeModeA(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeA, self).setUp()
-        self.mode = 'a'
+        self.mode = "a"
 
     def test_open_close_new_file(self):
         self.check_open_close_new_file()
@@ -484,7 +497,7 @@ class TestFakeModeA(FakeStatTestBase):
         self.check_open_write_flush_close_non_w_mode()
 
     def test_read_raises(self):
-        with self.open(self.file_path, 'a') as f:
+        with self.open(self.file_path, "a") as f:
             with self.assertRaises(OSError):
                 f.read()
 
@@ -497,7 +510,7 @@ class TestRealModeA(TestFakeModeA):
 class TestFakeModeAPlus(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeAPlus, self).setUp()
-        self.mode = 'a+'
+        self.mode = "a+"
 
     def test_open_close_new_file(self):
         self.check_open_close_new_file()
@@ -532,7 +545,7 @@ class TestRealModeAPlus(TestFakeModeAPlus):
 class TestFakeModeR(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeR, self).setUp()
-        self.mode = 'r'
+        self.mode = "r"
 
     def test_open_close(self):
         self.check_open_close_non_w_mode()
@@ -567,7 +580,7 @@ class TestFakeModeR(FakeStatTestBase):
 
     def test_open_not_existing_raises(self):
         with self.assertRaises(OSError):
-            with self.open(self.file_path, 'r'):
+            with self.open(self.file_path, "r"):
                 pass
 
 
@@ -579,7 +592,7 @@ class TestRealModeR(TestFakeModeR):
 class TestFakeModeRPlus(FakeStatTestBase):
     def setUp(self):
         super(TestFakeModeRPlus, self).setUp()
-        self.mode = 'r+'
+        self.mode = "r+"
 
     def test_open_close(self):
         self.check_open_close_non_w_mode()
@@ -598,7 +611,7 @@ class TestFakeModeRPlus(FakeStatTestBase):
 
     def test_open_not_existing_raises(self):
         with self.assertRaises(OSError):
-            with self.open(self.file_path, 'r+'):
+            with self.open(self.file_path, "r+"):
                 pass
 
 
@@ -607,5 +620,5 @@ class TestRealModeRPlus(TestFakeModeRPlus):
         return True
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
